@@ -34,19 +34,17 @@ enum {
     PROP_BASE,
 };
 
-static GType number_format_type;
-
 struct MpSerializerPrivate
 {
     gint accuracy;            /* Number of digits to show */
-    DisplayFormat format;     /* Number display mode. */
+    MpDisplayFormat format;   /* Number display mode. */
     gint show_tsep;           /* Set if the thousands separator should be shown. */
     gint show_zeroes;         /* Set if trailing zeroes should be shown. */
 
     gint base;                /* Numeric base */
 
-    char *tsep;         /* Locale specific thousands separator. */
-    char *radix;        /* Locale specific radix string. */
+    gunichar tsep;            /* Locale specific thousands separator. */
+    gunichar radix;           /* Locale specific radix string. */
     gint tsep_count;          /* Number of digits between separator. */
 };
 
@@ -54,15 +52,20 @@ struct MpSerializerPrivate
 G_DEFINE_TYPE(MpSerializer, mp_serializer, G_TYPE_OBJECT);
 
 MpSerializer *
-mp_serializer_new()
+mp_serializer_new(MpDisplayFormat format, int base, int accuracy)
 {
-    return g_object_new(mp_serializer_get_type(), NULL);
+    MpSerializer *serializer = g_object_new(mp_serializer_get_type(), /*"number-format", format,*/ NULL);
+    mp_serializer_set_number_format(serializer, format);
+    mp_serializer_set_base(serializer, base);
+    mp_serializer_set_accuracy(serializer, accuracy);
+    return serializer;
 }
 
+
 static void
-mp_cast_to_string_real(MpSerializer *serializer, const MPNumber *x, int base, bool force_sign, GString *string)
+mp_cast_to_string_real(MpSerializer *serializer, const MPNumber *x, int base, gboolean force_sign, GString *string)
 {
-    static char digits[] = "0123456789ABCDEF";
+    static gchar digits[] = "0123456789ABCDEF";
     MPNumber number, integer_component, fractional_component, temp;
     int i, last_non_zero;
 
@@ -103,15 +106,17 @@ mp_cast_to_string_real(MpSerializer *serializer, const MPNumber *x, int base, bo
         g_string_prepend_c(string, d < 16 ? digits[d] : '?');
 
         i++;
-        if (serializer->priv->show_tsep && i == serializer->priv->tsep_count)
-            g_string_prepend(string, serializer->priv->tsep);
+        if (serializer->priv->show_tsep && i == serializer->priv->tsep_count) {
+            g_string_prepend_unichar(string, serializer->priv->tsep);
+            i = 0;
+        }
 
         mp_set_from_mp(&t, &temp);
     } while (!mp_is_zero(&temp));
 
     last_non_zero = string->len;
 
-    g_string_append(string, serializer->priv->radix);
+    g_string_append_unichar(string, serializer->priv->radix);
 
     /* Write out the fractional component */
     mp_set_from_mp(&fractional_component, &temp);
@@ -144,7 +149,7 @@ mp_cast_to_string_real(MpSerializer *serializer, const MPNumber *x, int base, bo
 
     /* Append base suffix if not in default base */
     if (base != serializer->priv->base) {
-        const char *digits[] = {"₀", "₁", "₂", "₃", "₄", "₅", "₆", "₇", "₈", "₉"};
+        const gchar *digits[] = {"₀", "₁", "₂", "₃", "₄", "₅", "₆", "₇", "₈", "₉"};
         int multiplier = 1;
         int b = base;
 
@@ -161,26 +166,27 @@ mp_cast_to_string_real(MpSerializer *serializer, const MPNumber *x, int base, bo
 }
 
 
-static void
-mp_cast_to_string(MpSerializer *serializer, const MPNumber *x, char **buffer)
+static gchar *
+mp_cast_to_string(MpSerializer *serializer, const MPNumber *x)
 {
     GString *string;
     MPNumber x_real;
+    gchar *result;
 
     string = g_string_sized_new(1024);
 
     mp_real_component(x, &x_real);
-    mp_cast_to_string_real(serializer, &x_real, serializer->priv->base, false, string);
+    mp_cast_to_string_real(serializer, &x_real, serializer->priv->base, FALSE, string);
     if (mp_is_complex(x)) {
         GString *s;
-        gboolean force_sign = true;
+        gboolean force_sign = TRUE;
         MPNumber x_im;
 
         mp_imaginary_component(x, &x_im);
 
         if (strcmp(string->str, "0") == 0) {
             g_string_assign(string, "");
-            force_sign = false;
+            force_sign = FALSE;
         }
 
         s = g_string_sized_new(1024);
@@ -208,21 +214,19 @@ mp_cast_to_string(MpSerializer *serializer, const MPNumber *x, char **buffer)
         g_string_free(s, TRUE);
     }
 
-    *buffer = g_strndup(string->str, string->len + 1);
+    result = g_strndup(string->str, string->len + 1);
     g_string_free(string, TRUE);
+
+    return result;
 }
 
 
-static void
-mp_cast_to_exponential_string(MpSerializer *serializer, const MPNumber *x, bool eng_format, char **buffer)
+static int
+mp_cast_to_exponential_string_real(MpSerializer *serializer, const MPNumber *x, GString *string, gboolean eng_format)
 {
-    char *fixed, *c;
+    gchar *fixed;
     MPNumber t, z, base, base3, base10, base10inv, mantissa;
     int exponent = 0;
-    GString *string;
-    const char *super_digits[] = {"⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹"};
-
-    string = g_string_sized_new(1024);
 
     mp_abs(x, &z);
     if (mp_is_negative(x))
@@ -259,96 +263,181 @@ mp_cast_to_exponential_string(MpSerializer *serializer, const MPNumber *x, bool
         }
     }
 
-    mp_cast_to_string(serializer, &mantissa, &fixed);
+    fixed = mp_cast_to_string(serializer, &mantissa);
     g_string_append(string, fixed);
     g_free(fixed);
-    if (exponent != 0) {
-        g_string_append_printf(string, "×10"); // FIXME: Use the current base
-        if (exponent < 0) {
-            exponent = -exponent;
-            g_string_append(string, "⁻");
+  
+    return exponent;
+}
+
+
+static void
+append_exponent(GString *string, int exponent)
+{
+    const gchar *super_digits[] = {"⁰", "¹", "²", "³", "⁴", "⁵", "⁶", "⁷", "⁸", "⁹"};
+    gchar *super_value, *c;
+  
+    if (exponent == 0)
+        return;
+
+    g_string_append_printf(string, "×10"); // FIXME: Use the current base
+    if (exponent < 0) {
+        exponent = -exponent;
+        g_string_append(string, "⁻");
+    }
+
+    super_value = g_strdup_printf("%d", exponent);
+    for (c = super_value; *c; c++)
+        g_string_append(string, super_digits[*c - '0']);
+    g_free (super_value);
+}
+
+
+static gchar *
+mp_cast_to_exponential_string(MpSerializer *serializer, const MPNumber *x, gboolean eng_format)
+{
+    GString *string;
+    MPNumber x_real;
+    gchar *result;
+    int exponent;
+
+    string = g_string_sized_new(1024);
+
+    mp_real_component(x, &x_real);
+    exponent = mp_cast_to_exponential_string_real(serializer, &x_real, string, eng_format);
+    append_exponent(string, exponent);
+
+    if (mp_is_complex(x)) {
+        GString *s;
+        gboolean force_sign = TRUE;
+        MPNumber x_im;
+
+        mp_imaginary_component(x, &x_im);
+
+        if (strcmp(string->str, "0") == 0) {
+            g_string_assign(string, "");
+            force_sign = FALSE;
         }
-        snprintf(fixed, 1024, "%d", exponent);
-        for (c = fixed; *c; c++)
-            g_string_append(string, super_digits[*c - '0']);
+
+        s = g_string_sized_new(1024);
+        exponent = mp_cast_to_exponential_string_real(serializer, &x_im, s, eng_format);
+        if (strcmp(s->str, "0") == 0 || strcmp(s->str, "+0") == 0 || strcmp(s->str, "−0") == 0) {
+            /* Ignore */
+        }
+        else if (strcmp(s->str, "1") == 0) {
+            g_string_append(string, "i");
+        }
+        else if (strcmp(s->str, "+1") == 0) {
+            g_string_append(string, "+i");
+        }
+        else if (strcmp(s->str, "−1") == 0) {
+            g_string_append(string, "−i");
+        }
+        else {
+            if (strcmp(s->str, "+0") == 0)
+                g_string_append(string, "+");
+            else if (strcmp(s->str, "0") != 0)
+                g_string_append(string, s->str);
+
+            g_string_append(string, "i");
+        }
+        g_string_free(s, TRUE);
+        append_exponent(string, exponent);
     }
 
-    *buffer = g_strndup(string->str, string->len + 1);
+    result = g_strndup(string->str, string->len + 1);
     g_string_free(string, TRUE);
+
+    return result;
 }
 
 
-void
-mp_serializer_to_standard_string(MpSerializer *serializer, const MPNumber *x, char **target)
+gchar *
+mp_serializer_to_string(MpSerializer *serializer, const MPNumber *x)
 {
+    gchar *s0, *s1;
     switch(serializer->priv->format) {
-    case FIX:
-        mp_cast_to_string(serializer, x, target);
-        break;
-    case SCI:
-        mp_cast_to_exponential_string(serializer, x, false, target);
-        break;
-    case ENG:
-        mp_cast_to_exponential_string(serializer, x, true, target);
+    default:
+    case MP_DISPLAY_FORMAT_AUTOMATIC:
+        s0 = mp_cast_to_string(serializer, x);
+        s1 = mp_cast_to_exponential_string(serializer, x, FALSE);
+        if (g_utf8_strlen(s0, -1) < g_utf8_strlen(s1, -1))
+        {
+            g_free(s1);
+            return s0;
+        }
+        else
+        {
+            g_free(s0);
+            return s1;
+        }
         break;
+    case MP_DISPLAY_FORMAT_FIXED:
+        return mp_cast_to_string(serializer, x);
+    case MP_DISPLAY_FORMAT_SCIENTIFIC:
+        return mp_cast_to_exponential_string(serializer, x, FALSE);
+    case MP_DISPLAY_FORMAT_ENGINEERING:
+        return mp_cast_to_exponential_string(serializer, x, TRUE);
     }
 }
 
-void
-mp_serializer_to_specific_string(const MPNumber *x, int base, int accuracy, bool trim_zeroes, bool localize, char **target)
-{
-    MpSerializer *serializer = mp_serializer_new ();
-    if (!localize) {
-        g_free(serializer->priv->radix);
-        serializer->priv->radix = g_strdup(".");
-        serializer->priv->show_tsep = false;
-    }
-    serializer->priv->base = base;
-    serializer->priv->accuracy = accuracy;
-    serializer->priv->show_zeroes = !trim_zeroes;
-    mp_serializer_to_standard_string(serializer, x, target);
-    g_object_unref(serializer);
-}
 
-bool
-mp_serializer_from_string(MpSerializer *serializer, const char *str, MPNumber *z)
+gboolean
+mp_serializer_from_string(MpSerializer *serializer, const gchar *str, MPNumber *z)
 {
     return mp_set_from_string(str, serializer->priv->base, z);
 }
 
+
 void
 mp_serializer_set_base(MpSerializer *serializer, gint base)
 {
     serializer->priv->base = base;
 }
 
+
 int
 mp_serializer_get_base(MpSerializer *serializer)
 {
     return serializer->priv->base;
 }
 
-const gchar *
-mp_serializer_get_numeric_point_text(MpSerializer *serializer)
+
+void
+mp_serializer_set_radix(MpSerializer *serializer, gunichar radix)
+{
+    serializer->priv->radix = radix;
+}
+
+
+gunichar
+mp_serializer_get_radix(MpSerializer *serializer)
 {
     return serializer->priv->radix;
 }
 
 
-const gchar *
-mp_serializer_get_thousands_separator_text(MpSerializer *serializer)
+void
+mp_serializer_set_thousands_separator(MpSerializer *serializer, gunichar separator)
+{
+    serializer->priv->tsep = separator;
+}
+
+
+gunichar
+mp_serializer_get_thousands_separator(MpSerializer *serializer)
 {
     return serializer->priv->tsep;
 }
 
-gboolean
-mp_serializer_is_numeric_point(MpSerializer *serializer, const char *text)
+
+gint
+mp_serializer_get_thousands_separator_count(MpSerializer *serializer)
 {
-    if (*text == '.' || strncmp(text, serializer->priv->radix, strlen(serializer->priv->radix)) == 0)
-        return true;
-    return false;
+    return serializer->priv->tsep_count;
 }
 
+
 void
 mp_serializer_set_show_thousands_separators(MpSerializer *serializer, gboolean visible)
 {
@@ -383,29 +472,33 @@ mp_serializer_get_accuracy(MpSerializer *serializer)
     return serializer->priv->accuracy;
 }
 
+
 void
 mp_serializer_set_accuracy(MpSerializer *serializer, int accuracy)
 {
     serializer->priv->accuracy = accuracy;
 }
 
-DisplayFormat
+
+MpDisplayFormat
 mp_serializer_get_number_format(MpSerializer *serializer)
 {
     return serializer->priv->format;
 }
 
+
 void
-mp_serializer_set_number_format(MpSerializer *serializer, DisplayFormat format)
+mp_serializer_set_number_format(MpSerializer *serializer, MpDisplayFormat format)
 {
     serializer->priv->format = format;
 }
 
+
 static void
 mp_serializer_set_property(GObject *object,
-                          guint prop_id,
-                          const GValue *value,
-                          GParamSpec *pspec)
+                           guint prop_id,
+                           const GValue *value,
+                           GParamSpec *pspec)
 {
     MpSerializer *self = MP_SERIALIZER(object);
 
@@ -425,11 +518,12 @@ mp_serializer_set_property(GObject *object,
     }
 }
 
+
 static void
 mp_serializer_get_property(GObject *object,
-                          guint prop_id,
-                          GValue *value,
-                          GParamSpec *pspec)
+                           guint prop_id,
+                           GValue *value,
+                           GParamSpec *pspec)
 {
     MpSerializer *self = MP_SERIALIZER(object);
 
@@ -444,30 +538,20 @@ mp_serializer_get_property(GObject *object,
         g_value_set_int(value, mp_serializer_get_base(self));
         break;
     default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
 }
 
-static void
-mp_serializer_finalize (GObject *gobject)
-{
-    MpSerializer *serializer = MP_SERIALIZER(gobject);
-    g_free(serializer->priv->radix);
-    g_free(serializer->priv->tsep);
-}
 
 static void
-mp_serializer_class_init (MpSerializerClass *klass)
+mp_serializer_class_init(MpSerializerClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS(klass);
     object_class->get_property = mp_serializer_get_property;
     object_class->set_property = mp_serializer_set_property;
-    object_class->finalize = mp_serializer_finalize;
 
-    g_type_class_add_private (klass, sizeof(MpSerializerPrivate));
-
-    number_format_type = math_display_format_get_type();
+    g_type_class_add_private(klass, sizeof(MpSerializerPrivate));
 
     g_object_class_install_property(object_class,
                                     PROP_SHOW_THOUSANDS_SEPARATORS,
@@ -483,23 +567,24 @@ mp_serializer_class_init (MpSerializerClass *klass)
                                                          "Show trailing zeroes",
                                                          FALSE,
                                                          G_PARAM_READWRITE));
-    g_object_class_install_property(object_class,
-                                    PROP_BASE,
-                                    g_param_spec_int("base",
-                                                     "base",
-                                                     "Default number base (derived from number-format)",
-                                                     2, 16, 10, 
-                                                     G_PARAM_READWRITE));
     g_object_class_install_property(object_class,
                                     PROP_NUMBER_FORMAT,
                                     g_param_spec_enum("number-format",
                                                       "number-format",
                                                       "Display format",
-                                                      number_format_type,
-                                                      FIX,
+                                                      math_mp_display_format_get_type(),
+                                                      MP_DISPLAY_FORMAT_AUTOMATIC,
                                                       G_PARAM_READWRITE));
+    g_object_class_install_property(object_class,
+                                    PROP_BASE,
+                                    g_param_spec_int("base",
+                                                     "base",
+                                                     "Default number base",
+                                                     2, 16, 10, 
+                                                     G_PARAM_READWRITE));
 }
 
+
 static void
 mp_serializer_init(MpSerializer *serializer)
 {
@@ -507,14 +592,17 @@ mp_serializer_init(MpSerializer *serializer)
     serializer->priv = G_TYPE_INSTANCE_GET_PRIVATE(serializer, mp_serializer_get_type(), MpSerializerPrivate);
 
     radix = nl_langinfo(RADIXCHAR);
-    serializer->priv->radix = *radix ? g_locale_to_utf8(radix, -1, NULL, NULL, NULL) : g_strdup(".");
+    serializer->priv->radix = radix ? g_utf8_get_char(g_locale_to_utf8(radix, -1, NULL, NULL, NULL)) : '.';
     tsep = nl_langinfo(THOUSEP);
-    serializer->priv->tsep = *tsep ? g_locale_to_utf8(tsep, -1, NULL, NULL, NULL) : g_strdup("");
-
+    if (tsep && tsep[0] != '\0')
+        serializer->priv->tsep = g_utf8_get_char(g_locale_to_utf8(tsep, -1, NULL, NULL, NULL));
+    else
+        serializer->priv->tsep = ' ';
     serializer->priv->tsep_count = 3;
+
     serializer->priv->base = 10;
     serializer->priv->accuracy = 9;
     serializer->priv->show_zeroes = FALSE;
     serializer->priv->show_tsep = FALSE;
-    serializer->priv->format = FIX;
+    serializer->priv->format = MP_DISPLAY_FORMAT_AUTOMATIC;
 }
