@@ -23,7 +23,10 @@
 #include <errno.h>
 
 #include "mp.h"
-#include "mp-internal.h"
+#include "mp-private.h"
+
+static MPNumber eulers_number;
+static gboolean have_eulers_number = FALSE;
 
 // FIXME: Re-add overflow and underflow detection
 
@@ -101,9 +104,13 @@ mp_ext(int i, int j, MPNumber *x)
 void
 mp_get_eulers(MPNumber *z)
 {
-    MPNumber t;
-    mp_set_from_integer(1, &t);
-    mp_epowy(&t, z);
+    if (!have_eulers_number) {
+        MPNumber t;
+        mp_set_from_integer(1, &t);
+        mp_epowy(&t, &eulers_number);
+        have_eulers_number = TRUE;
+    }
+    mp_set_from_mp(&eulers_number, z);
 }
 
 
@@ -293,6 +300,15 @@ mp_add_real(const MPNumber *x, int y_sign, const MPNumber *y, MPNumber *z)
         z->fraction[MP_T + i] = 0;
 
     if (sign_prod >= 0) {
+        /* This is probably insufficient overflow detection, but it makes us
+         * not crash at least.
+         */
+        if (MP_T + 3 < med) {
+            mperr(_("Overflow: the result couldn't be calculated"));
+            mp_set_from_integer(0, z);
+            return;
+        }
+
         /* HERE DO ADDITION, EXPONENT(Y) >= EXPONENT(X) */
         for (i = MP_T + 3; i >= MP_T; i--)
             z->fraction[i] = small_fraction[i - med];
@@ -1767,7 +1783,7 @@ mp_reciprocal(const MPNumber *x, MPNumber *z)
 }
 
 
-void
+static void
 mp_root_real(const MPNumber *x, int64_t n, MPNumber *z)
 {
     float approximation;
